import typing
import datetime as _datetime

from kubernetes import client
from kuber import kube_api as _kube_api

from kuber import definitions as _kuber_definitions
from kuber.v1_11.meta_v1 import ListMeta
from kuber.v1_11.meta_v1 import ObjectMeta
from kuber.v1_11.meta_v1 import Status
from kuber.v1_11.meta_v1 import StatusDetails


class CertificateSigningRequest(_kuber_definitions.Resource):
    """
    Describes a certificate signing request
    """

    def __init__(
            self,
            metadata: 'ObjectMeta' = None,
            spec: 'CertificateSigningRequestSpec' = None,
            status: 'CertificateSigningRequestStatus' = None,
    ):
        """Create CertificateSigningRequest instance."""
        super(CertificateSigningRequest, self).__init__(
            api_version='certificates/v1beta1',
            kind='CertificateSigningRequest'
        )
        self._properties = {
            'metadata': metadata if metadata is not None else ObjectMeta(),
            'spec': spec if spec is not None else CertificateSigningRequestSpec(),
            'status': status if status is not None else CertificateSigningRequestStatus(),

        }
        self._types = {
            'apiVersion': (str, None),
            'kind': (str, None),
            'metadata': (ObjectMeta, None),
            'spec': (CertificateSigningRequestSpec, None),
            'status': (CertificateSigningRequestStatus, None),

        }

    @property
    def metadata(self) -> 'ObjectMeta':
        """

        """
        return self._properties.get('metadata')

    @metadata.setter
    def metadata(self, value: typing.Union['ObjectMeta', dict]):
        """

        """
        if isinstance(value, dict):
            value = ObjectMeta().from_dict(value)
        self._properties['metadata'] = value

    @property
    def spec(self) -> 'CertificateSigningRequestSpec':
        """
        The certificate request itself and any additional
        information.
        """
        return self._properties.get('spec')

    @spec.setter
    def spec(self, value: typing.Union['CertificateSigningRequestSpec', dict]):
        """
        The certificate request itself and any additional
        information.
        """
        if isinstance(value, dict):
            value = CertificateSigningRequestSpec().from_dict(value)
        self._properties['spec'] = value

    @property
    def status(self) -> 'CertificateSigningRequestStatus':
        """
        Derived information about the request.
        """
        return self._properties.get('status')

    @status.setter
    def status(self, value: typing.Union['CertificateSigningRequestStatus', dict]):
        """
        Derived information about the request.
        """
        if isinstance(value, dict):
            value = CertificateSigningRequestStatus().from_dict(value)
        self._properties['status'] = value

    def create_resource(
            self,
            namespace: 'str' = None
    ) -> 'CertificateSigningRequestStatus':
        """
        Creates the CertificateSigningRequest in the currently
        configured Kubernetes cluster and returns the status information
        returned by the Kubernetes API after the create is complete.
        """
        names = [
            'create_namespaced_certificate_signing_request',
            'create_certificate_signing_request'
        ]

        response = _kube_api.execute(
            action='create',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'body': self.to_dict()}
        )
        return (
            CertificateSigningRequestStatus()
            .from_dict(_kube_api.to_kuber_dict(response.status))
        )

    def replace_resource(
            self,
            namespace: 'str' = None
    ) -> 'CertificateSigningRequestStatus':
        """
        Replaces the CertificateSigningRequest in the currently
        configured Kubernetes cluster and returns the status information
        returned by the Kubernetes API after the replace is complete.
        """
        names = [
            'replace_namespaced_certificate_signing_request',
            'replace_certificate_signing_request'
        ]

        response = _kube_api.execute(
            action='replace',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'body': self.to_dict(), 'name': self.metadata.name}
        )
        return (
            CertificateSigningRequestStatus()
            .from_dict(_kube_api.to_kuber_dict(response.status))
        )

    def patch_resource(
            self,
            namespace: 'str' = None
    ) -> 'CertificateSigningRequestStatus':
        """
        Patches the CertificateSigningRequest in the currently
        configured Kubernetes cluster and returns the status information
        returned by the Kubernetes API after the replace is complete.
        """
        names = [
            'patch_namespaced_certificate_signing_request',
            'patch_certificate_signing_request'
        ]

        response = _kube_api.execute(
            action='patch',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'body': self.to_dict(), 'name': self.metadata.name}
        )
        return (
            CertificateSigningRequestStatus()
            .from_dict(_kube_api.to_kuber_dict(response.status))
        )

    def get_resource_status(
            self,
            namespace: 'str' = None
    ) -> 'CertificateSigningRequestStatus':
        """
        Returns status information about the given resource within the cluster.
        """
        names = [
            'read_namespaced_certificate_signing_request',
            'read_certificate_signing_request'
        ]

        response = _kube_api.execute(
            action='read',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'name': self.metadata.name}
        )
        return (
            CertificateSigningRequestStatus()
            .from_dict(_kube_api.to_kuber_dict(response.status))
        )

    def read_resource(
            self,
            namespace: str = None
    ):
        """
        Reads the CertificateSigningRequest from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            'read_namespaced_certificate_signing_request',
            'read_certificate_signing_request'
        ]
        return _kube_api.execute(
            action='read',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'name': self.metadata.name}
        )

    def delete_resource(
            self,
            namespace: str = None,
            propagation_policy: str = 'Foreground',
            grace_period_seconds: int = 10
    ):
        """
        Deletes the CertificateSigningRequest from the currently configured
        Kubernetes cluster.
        """
        names = [
            'delete_namespaced_certificate_signing_request',
            'delete_certificate_signing_request'
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds
        )

        _kube_api.execute(
            action='delete',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'name': self.metadata.name, 'body': body}
        )

    @staticmethod
    def get_resource_api(
            api_client: client.ApiClient = None,
            **kwargs
    ) -> 'client.CertificatesV1beta1Api':
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs['apl_client'] = api_client
        return client.CertificatesV1beta1Api(**kwargs)

    def __enter__(self) -> 'CertificateSigningRequest':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class CertificateSigningRequestCondition(_kuber_definitions.Definition):
    """

    """

    def __init__(
            self,
            last_update_time: str = None,
            message: str = None,
            reason: str = None,
            type_: str = None,
    ):
        """Create CertificateSigningRequestCondition instance."""
        super(CertificateSigningRequestCondition, self).__init__(
            api_version='certificates/v1beta1',
            kind='CertificateSigningRequestCondition'
        )
        self._properties = {
            'lastUpdateTime': last_update_time if last_update_time is not None else None,
            'message': message if message is not None else '',
            'reason': reason if reason is not None else '',
            'type': type_ if type_ is not None else '',

        }
        self._types = {
            'lastUpdateTime': (str, None),
            'message': (str, None),
            'reason': (str, None),
            'type': (str, None),

        }

    @property
    def last_update_time(self) -> str:
        """
        timestamp for the last update to this condition
        """
        return self._properties.get('lastUpdateTime')

    @last_update_time.setter
    def last_update_time(
            self,
            value: typing.Union[str, _datetime.datetime, _datetime.date]
    ):
        """
        timestamp for the last update to this condition
        """
        if isinstance(value, _datetime.datetime):
            value = value.strftime('%Y-%m-%dT%H:%M:%SZ')
        elif isinstance(value, _datetime.date):
            value = value.strftime('%Y-%m-%dT00:00:00Z')
        self._properties['lastUpdateTime'] = value

    @property
    def message(self) -> str:
        """
        human readable message with details about the request state
        """
        return self._properties.get('message')

    @message.setter
    def message(self, value: str):
        """
        human readable message with details about the request state
        """
        self._properties['message'] = value

    @property
    def reason(self) -> str:
        """
        brief reason for the request state
        """
        return self._properties.get('reason')

    @reason.setter
    def reason(self, value: str):
        """
        brief reason for the request state
        """
        self._properties['reason'] = value

    @property
    def type_(self) -> str:
        """
        request approval state, currently Approved or Denied.
        """
        return self._properties.get('type')

    @type_.setter
    def type_(self, value: str):
        """
        request approval state, currently Approved or Denied.
        """
        self._properties['type'] = value

    def __enter__(self) -> 'CertificateSigningRequestCondition':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class CertificateSigningRequestList(_kuber_definitions.Collection):
    """

    """

    def __init__(
            self,
            items: typing.List['CertificateSigningRequest'] = None,
            metadata: 'ListMeta' = None,
    ):
        """Create CertificateSigningRequestList instance."""
        super(CertificateSigningRequestList, self).__init__(
            api_version='certificates/v1beta1',
            kind='CertificateSigningRequestList'
        )
        self._properties = {
            'items': items if items is not None else [],
            'metadata': metadata if metadata is not None else ListMeta(),

        }
        self._types = {
            'apiVersion': (str, None),
            'items': (list, CertificateSigningRequest),
            'kind': (str, None),
            'metadata': (ListMeta, None),

        }

    @property
    def items(self) -> typing.List['CertificateSigningRequest']:
        """

        """
        return self._properties.get('items')

    @items.setter
    def items(
            self,
            value: typing.Union[typing.List['CertificateSigningRequest'], typing.List[dict]]
    ):
        """

        """
        cleaned = []
        for item in value:
            if isinstance(item, dict):
                item = CertificateSigningRequest().from_dict(item)
            cleaned.append(item)
        self._properties['items'] = cleaned

    @property
    def metadata(self) -> 'ListMeta':
        """

        """
        return self._properties.get('metadata')

    @metadata.setter
    def metadata(self, value: typing.Union['ListMeta', dict]):
        """

        """
        if isinstance(value, dict):
            value = ListMeta().from_dict(value)
        self._properties['metadata'] = value

    @staticmethod
    def get_resource_api(
            api_client: client.ApiClient = None,
            **kwargs
    ) -> 'client.CertificatesV1beta1Api':
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs['apl_client'] = api_client
        return client.CertificatesV1beta1Api(**kwargs)

    def __enter__(self) -> 'CertificateSigningRequestList':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class CertificateSigningRequestSpec(_kuber_definitions.Definition):
    """
    This information is immutable after the request is created.
    Only the Request and Usages fields can be set on creation,
    other fields are derived by Kubernetes and cannot be
    modified by users.
    """

    def __init__(
            self,
            extra: dict = None,
            groups: typing.List[str] = None,
            request: str = None,
            uid: str = None,
            usages: typing.List[str] = None,
            username: str = None,
    ):
        """Create CertificateSigningRequestSpec instance."""
        super(CertificateSigningRequestSpec, self).__init__(
            api_version='certificates/v1beta1',
            kind='CertificateSigningRequestSpec'
        )
        self._properties = {
            'extra': extra if extra is not None else {},
            'groups': groups if groups is not None else [],
            'request': request if request is not None else '',
            'uid': uid if uid is not None else '',
            'usages': usages if usages is not None else [],
            'username': username if username is not None else '',

        }
        self._types = {
            'extra': (dict, None),
            'groups': (list, str),
            'request': (str, None),
            'uid': (str, None),
            'usages': (list, str),
            'username': (str, None),

        }

    @property
    def extra(self) -> dict:
        """
        Extra information about the requesting user. See user.Info
        interface for details.
        """
        return self._properties.get('extra')

    @extra.setter
    def extra(self, value: dict):
        """
        Extra information about the requesting user. See user.Info
        interface for details.
        """
        self._properties['extra'] = value

    @property
    def groups(self) -> typing.List[str]:
        """
        Group information about the requesting user. See user.Info
        interface for details.
        """
        return self._properties.get('groups')

    @groups.setter
    def groups(self, value: typing.List[str]):
        """
        Group information about the requesting user. See user.Info
        interface for details.
        """
        self._properties['groups'] = value

    @property
    def request(self) -> str:
        """
        Base64-encoded PKCS#10 CSR data
        """
        return self._properties.get('request')

    @request.setter
    def request(self, value: str):
        """
        Base64-encoded PKCS#10 CSR data
        """
        self._properties['request'] = value

    @property
    def uid(self) -> str:
        """
        UID information about the requesting user. See user.Info
        interface for details.
        """
        return self._properties.get('uid')

    @uid.setter
    def uid(self, value: str):
        """
        UID information about the requesting user. See user.Info
        interface for details.
        """
        self._properties['uid'] = value

    @property
    def usages(self) -> typing.List[str]:
        """
        allowedUsages specifies a set of usage contexts the key will
        be valid for. See:
        https://tools.ietf.org/html/rfc5280#section-4.2.1.3
             https://tools.ietf.org/html/rfc5280#section-4.2.1.12
        """
        return self._properties.get('usages')

    @usages.setter
    def usages(self, value: typing.List[str]):
        """
        allowedUsages specifies a set of usage contexts the key will
        be valid for. See:
        https://tools.ietf.org/html/rfc5280#section-4.2.1.3
             https://tools.ietf.org/html/rfc5280#section-4.2.1.12
        """
        self._properties['usages'] = value

    @property
    def username(self) -> str:
        """
        Information about the requesting user. See user.Info
        interface for details.
        """
        return self._properties.get('username')

    @username.setter
    def username(self, value: str):
        """
        Information about the requesting user. See user.Info
        interface for details.
        """
        self._properties['username'] = value

    def __enter__(self) -> 'CertificateSigningRequestSpec':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class CertificateSigningRequestStatus(_kuber_definitions.Definition):
    """

    """

    def __init__(
            self,
            certificate: str = None,
            conditions: typing.List['CertificateSigningRequestCondition'] = None,
    ):
        """Create CertificateSigningRequestStatus instance."""
        super(CertificateSigningRequestStatus, self).__init__(
            api_version='certificates/v1beta1',
            kind='CertificateSigningRequestStatus'
        )
        self._properties = {
            'certificate': certificate if certificate is not None else '',
            'conditions': conditions if conditions is not None else [],

        }
        self._types = {
            'certificate': (str, None),
            'conditions': (list, CertificateSigningRequestCondition),

        }

    @property
    def certificate(self) -> str:
        """
        If request was approved, the controller will place the
        issued certificate here.
        """
        return self._properties.get('certificate')

    @certificate.setter
    def certificate(self, value: str):
        """
        If request was approved, the controller will place the
        issued certificate here.
        """
        self._properties['certificate'] = value

    @property
    def conditions(self) -> typing.List['CertificateSigningRequestCondition']:
        """
        Conditions applied to the request, such as approval or
        denial.
        """
        return self._properties.get('conditions')

    @conditions.setter
    def conditions(
            self,
            value: typing.Union[typing.List['CertificateSigningRequestCondition'], typing.List[dict]]
    ):
        """
        Conditions applied to the request, such as approval or
        denial.
        """
        cleaned = []
        for item in value:
            if isinstance(item, dict):
                item = CertificateSigningRequestCondition().from_dict(item)
            cleaned.append(item)
        self._properties['conditions'] = cleaned

    def __enter__(self) -> 'CertificateSigningRequestStatus':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False



import os

import kuber
from kuber.latest import batch_v1beta1

MY_DIRECTORY = os.path.realpath(os.path.dirname(__file__))


def test_get_containers_from_job():
    """Should get containers without error."""
    path = os.path.join(MY_DIRECTORY, 'cronjob.yaml')
    cron_job: batch_v1beta1.CronJob = kuber.from_yaml_file(path)
    assert len(cron_job.get_containers()) == 1



from unittest.mock import MagicMock
from unittest.mock import patch

from kuber import execution
from kuber.tests import utils as test_utils


def test_parse_api_exception():
    """
    Should parse ApiException into a response info object when the exception
    body cannot be parsed.
    """
    resource = MagicMock()
    error = MagicMock()
    error.body = '{"reason": "foo"}'
    response = execution._parse_api_exception(resource, error)
    assert response.reason == 'foo'


def test_parse_api_exception_no_body():
    """
    Should parse ApiException into a response info object when the exception
    body cannot be parsed.
    """
    resource = MagicMock()
    error = MagicMock()
    error.body = '{"a'
    response = execution._parse_api_exception(resource, error)
    assert response.reason == 'UnknownError'


def test_echo_response_normal():
    """Should display message without error when response is normal."""
    response = execution.ResponseInfo(
        resource=MagicMock(),
        symbol='  ',
        reason='Success',
        message='foo'
    )
    execution._echo_response(response)


def test_echo_response_error():
    """Should display message without error when response is an error."""
    response = execution.ResponseInfo(
        resource=MagicMock(),
        symbol='!!',
        reason='Failure',
        message='Error occurred',
        exception=MagicMock()
    )
    execution._echo_response(response)


def test_create_resource():
    """Should create resource successfully when no error is caught."""
    response = execution.create_resource(MagicMock(), echo=True)
    assert response.symbol == '+'
    assert response.reason == 'Created'


def test_create_resource_error():
    """Should handle failed resource creation error."""
    resource = MagicMock()
    resource.create_resource.side_effect = test_utils.MockApiException(
        successful=False
    )
    response = execution.create_resource(resource, echo=True)

    resource.create_resource.assert_called_once()
    assert response.symbol == '!!'
    assert response.reason == 'Testing'


@patch('kuber.execution.patch_resource')
def test_create_resource_exists(patch_resource: MagicMock):
    """Should attempt to patch resource if it already exists."""
    resource = MagicMock()
    resource.create_resource.side_effect = test_utils.MockApiException(
        successful=False,
        reason='AlreadyExists'
    )
    execution.create_resource(resource, echo=True)

    resource.create_resource.assert_called_once()
    patch_resource.assert_called_once()


def test_replace_resource():
    """Should replace resource successfully when no error is caught."""
    response = execution.replace_resource(MagicMock(), echo=True)
    assert response.symbol == '-/+'
    assert response.reason == 'Replaced'


def test_replace_resource_error():
    """Should handle failed resource replacement error."""
    resource = MagicMock()
    resource.replace_resource.side_effect = test_utils.MockApiException(
        successful=False
    )
    response = execution.replace_resource(resource, echo=True)

    resource.replace_resource.assert_called_once()
    assert response.symbol == '!!'
    assert response.reason == 'Testing'


def test_patch_resource():
    """Should patch resource successfully when no error is caught."""
    response = execution.patch_resource(MagicMock(), echo=True)
    assert response.symbol == '~'
    assert response.reason == 'Updated'


def test_patch_resource_error():
    """Should handle failed resource patching error."""
    resource = MagicMock()
    resource.patch_resource.side_effect = test_utils.MockApiException(
        successful=False
    )
    response = execution.patch_resource(resource, echo=True)

    resource.patch_resource.assert_called_once()
    assert response.symbol == '!!'
    assert response.reason == 'Testing'


def test_delete_resource():
    """Should delete resource successfully when no error is caught."""
    response = execution.delete_resource(MagicMock(), echo=True)
    assert response.symbol == '-'
    assert response.reason == 'Deleted'


def test_delete_resource_error():
    """Should handle failed resource deletion error."""
    resource = MagicMock()
    resource.delete_resource.side_effect = test_utils.MockApiException(
        successful=False
    )
    response = execution.delete_resource(resource, echo=True)

    resource.delete_resource.assert_called_once()
    assert response.symbol == '!!'
    assert response.reason == 'Testing'


def test_get_resource_status_error():
    """Should handle failed resource status retrieval error."""
    resource = MagicMock()
    resource.get_resource_status.side_effect = test_utils.MockApiException(
        successful=False
    )
    response = execution.get_resource_status(resource, echo=True)

    resource.get_resource_status.assert_called_once()
    assert response.symbol == '!!'
    assert response.reason == 'Testing'


def test_get_resource_status():
    """Should successfully get status for resource."""
    status = MagicMock()
    status.to_dict.return_value = {'foo': 'bar'}
    resource = MagicMock()
    resource.get_resource_status.return_value = status
    response = execution.get_resource_status(resource, echo=True)

    resource.get_resource_status.assert_called_once()
    assert response.symbol == '*'
    assert response.reason == 'Status'
    assert 'foo' in response.message and 'bar' in response.message


def test_get_resource_status_no_status_object():
    """Should successfully get status for resource."""
    resource = MagicMock()
    resource.get_resource_status.return_value = None
    response = execution.get_resource_status(resource, echo=True)

    resource.get_resource_status.assert_called_once()
    assert response.symbol == '*'
    assert response.reason == 'Status'



import os

import kuber

MY_DIRECTORY = os.path.realpath(os.path.dirname(__file__))


def test_new():
    """Should create new resource, add it to bundle and return it."""
    bundle = kuber.create_bundle()
    resource = bundle.new('v1', 'ConfigMap', 'foo')
    assert resource == bundle.resources[-1]
    assert resource.metadata.name == 'foo'


def test_add_directory_files():
    """Should add first resources before second resources."""
    bundle = kuber.from_directory_files(
        directory=MY_DIRECTORY,
        filenames=['test-first.yaml', 'test-second.yaml']
    )
    names = [r.metadata.name for r in bundle.resources]
    assert names == ['first-a', 'first-b', 'second-a', 'second-b']


def test_add_directory_files_reversed():
    """Should add second resources before first resources."""
    bundle = kuber.from_directory_files(
        directory=MY_DIRECTORY,
        filenames=['test-second.yaml', 'test-first.yaml']
    )
    names = [r.metadata.name for r in bundle.resources]
    assert names == ['second-a', 'second-b', 'first-a', 'first-b']



import typing

from kubernetes import client

from kuber import definitions as _kuber_definitions


class Info(_kuber_definitions.Definition):
    """
    Info contains versioning information. how we'll want to
    distribute that information.
    """

    def __init__(
            self,
            build_date: str = None,
            compiler: str = None,
            git_commit: str = None,
            git_tree_state: str = None,
            git_version: str = None,
            go_version: str = None,
            major: str = None,
            minor: str = None,
            platform: str = None,
    ):
        """Create Info instance."""
        super(Info, self).__init__(
            api_version='apimachinery/version',
            kind='Info'
        )
        self._properties = {
            'buildDate': build_date if build_date is not None else '',
            'compiler': compiler if compiler is not None else '',
            'gitCommit': git_commit if git_commit is not None else '',
            'gitTreeState': git_tree_state if git_tree_state is not None else '',
            'gitVersion': git_version if git_version is not None else '',
            'goVersion': go_version if go_version is not None else '',
            'major': major if major is not None else '',
            'minor': minor if minor is not None else '',
            'platform': platform if platform is not None else '',

        }
        self._types = {
            'buildDate': (str, None),
            'compiler': (str, None),
            'gitCommit': (str, None),
            'gitTreeState': (str, None),
            'gitVersion': (str, None),
            'goVersion': (str, None),
            'major': (str, None),
            'minor': (str, None),
            'platform': (str, None),

        }

    @property
    def build_date(self) -> str:
        """

        """
        return self._properties.get('buildDate')

    @build_date.setter
    def build_date(self, value: str):
        """

        """
        self._properties['buildDate'] = value

    @property
    def compiler(self) -> str:
        """

        """
        return self._properties.get('compiler')

    @compiler.setter
    def compiler(self, value: str):
        """

        """
        self._properties['compiler'] = value

    @property
    def git_commit(self) -> str:
        """

        """
        return self._properties.get('gitCommit')

    @git_commit.setter
    def git_commit(self, value: str):
        """

        """
        self._properties['gitCommit'] = value

    @property
    def git_tree_state(self) -> str:
        """

        """
        return self._properties.get('gitTreeState')

    @git_tree_state.setter
    def git_tree_state(self, value: str):
        """

        """
        self._properties['gitTreeState'] = value

    @property
    def git_version(self) -> str:
        """

        """
        return self._properties.get('gitVersion')

    @git_version.setter
    def git_version(self, value: str):
        """

        """
        self._properties['gitVersion'] = value

    @property
    def go_version(self) -> str:
        """

        """
        return self._properties.get('goVersion')

    @go_version.setter
    def go_version(self, value: str):
        """

        """
        self._properties['goVersion'] = value

    @property
    def major(self) -> str:
        """

        """
        return self._properties.get('major')

    @major.setter
    def major(self, value: str):
        """

        """
        self._properties['major'] = value

    @property
    def minor(self) -> str:
        """

        """
        return self._properties.get('minor')

    @minor.setter
    def minor(self, value: str):
        """

        """
        self._properties['minor'] = value

    @property
    def platform(self) -> str:
        """

        """
        return self._properties.get('platform')

    @platform.setter
    def platform(self, value: str):
        """

        """
        self._properties['platform'] = value

    def __enter__(self) -> 'Info':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False



import json

from kubernetes.client.rest import ApiException as _ApiException


class MockApiException(_ApiException):
    """Mock ApiException class for use in testing."""

    def __init__(
            self,
            successful: bool = True,
            reason: str = 'Testing',
            body: dict = None
    ):
        super(MockApiException, self).__init__(
            reason=reason,
            status=200 if successful else 442
        )
        self.body = json.dumps({**{'reason': reason}, **(body or {})})



import typing

from kubernetes import client
from kuber import kube_api as _kube_api

from kuber import definitions as _kuber_definitions
from kuber.v1_12.meta_v1 import ListMeta
from kuber.v1_12.meta_v1 import MicroTime
from kuber.v1_12.meta_v1 import ObjectMeta


class Lease(_kuber_definitions.Resource):
    """
    Lease defines a lease concept.
    """

    def __init__(
            self,
            metadata: 'ObjectMeta' = None,
            spec: 'LeaseSpec' = None,
    ):
        """Create Lease instance."""
        super(Lease, self).__init__(
            api_version='coordination/v1beta1',
            kind='Lease'
        )
        self._properties = {
            'metadata': metadata if metadata is not None else ObjectMeta(),
            'spec': spec if spec is not None else LeaseSpec(),

        }
        self._types = {
            'apiVersion': (str, None),
            'kind': (str, None),
            'metadata': (ObjectMeta, None),
            'spec': (LeaseSpec, None),

        }

    @property
    def metadata(self) -> 'ObjectMeta':
        """
        More info:
        https://git.k8s.io/community/contributors/devel/api-
        conventions.md#metadata
        """
        return self._properties.get('metadata')

    @metadata.setter
    def metadata(self, value: typing.Union['ObjectMeta', dict]):
        """
        More info:
        https://git.k8s.io/community/contributors/devel/api-
        conventions.md#metadata
        """
        if isinstance(value, dict):
            value = ObjectMeta().from_dict(value)
        self._properties['metadata'] = value

    @property
    def spec(self) -> 'LeaseSpec':
        """
        Specification of the Lease. More info:
        https://git.k8s.io/community/contributors/devel/api-
        conventions.md#spec-and-status
        """
        return self._properties.get('spec')

    @spec.setter
    def spec(self, value: typing.Union['LeaseSpec', dict]):
        """
        Specification of the Lease. More info:
        https://git.k8s.io/community/contributors/devel/api-
        conventions.md#spec-and-status
        """
        if isinstance(value, dict):
            value = LeaseSpec().from_dict(value)
        self._properties['spec'] = value

    def create_resource(self, namespace: 'str' = None):
        """
        Creates the Lease in the currently
        configured Kubernetes cluster.
        """
        names = [
            'create_namespaced_lease',
            'create_lease'
        ]

        _kube_api.execute(
            action='create',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'body': self.to_dict()}
        )

    def replace_resource(self, namespace: 'str' = None):
        """
        Replaces the Lease in the currently
        configured Kubernetes cluster.
        """
        names = [
            'replace_namespaced_lease',
            'replace_lease'
        ]

        _kube_api.execute(
            action='replace',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'body': self.to_dict(), 'name': self.metadata.name}
        )

    def patch_resource(self, namespace: 'str' = None):
        """
        Patches the Lease in the currently
        configured Kubernetes cluster.
        """
        names = [
            'patch_namespaced_lease',
            'patch_lease'
        ]

        _kube_api.execute(
            action='patch',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'body': self.to_dict(), 'name': self.metadata.name}
        )

    def get_resource_status(self, namespace: 'str' = None):
        """This resource does not have a status."""
        pass

    def read_resource(
            self,
            namespace: str = None
    ):
        """
        Reads the Lease from the currently configured
        Kubernetes cluster and returns the low-level definition object.
        """
        names = [
            'read_namespaced_lease',
            'read_lease'
        ]
        return _kube_api.execute(
            action='read',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'name': self.metadata.name}
        )

    def delete_resource(
            self,
            namespace: str = None,
            propagation_policy: str = 'Foreground',
            grace_period_seconds: int = 10
    ):
        """
        Deletes the Lease from the currently configured
        Kubernetes cluster.
        """
        names = [
            'delete_namespaced_lease',
            'delete_lease'
        ]

        body = client.V1DeleteOptions(
            propagation_policy=propagation_policy,
            grace_period_seconds=grace_period_seconds
        )

        _kube_api.execute(
            action='delete',
            resource=self,
            names=names,
            namespace=namespace,
            api_client=None,
            api_args={'name': self.metadata.name, 'body': body}
        )

    @staticmethod
    def get_resource_api(
            api_client: client.ApiClient = None,
            **kwargs
    ) -> 'client.CoordinationV1beta1Api':
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs['apl_client'] = api_client
        return client.CoordinationV1beta1Api(**kwargs)

    def __enter__(self) -> 'Lease':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class LeaseList(_kuber_definitions.Collection):
    """
    LeaseList is a list of Lease objects.
    """

    def __init__(
            self,
            items: typing.List['Lease'] = None,
            metadata: 'ListMeta' = None,
    ):
        """Create LeaseList instance."""
        super(LeaseList, self).__init__(
            api_version='coordination/v1beta1',
            kind='LeaseList'
        )
        self._properties = {
            'items': items if items is not None else [],
            'metadata': metadata if metadata is not None else ListMeta(),

        }
        self._types = {
            'apiVersion': (str, None),
            'items': (list, Lease),
            'kind': (str, None),
            'metadata': (ListMeta, None),

        }

    @property
    def items(self) -> typing.List['Lease']:
        """
        Items is a list of schema objects.
        """
        return self._properties.get('items')

    @items.setter
    def items(
            self,
            value: typing.Union[typing.List['Lease'], typing.List[dict]]
    ):
        """
        Items is a list of schema objects.
        """
        cleaned = []
        for item in value:
            if isinstance(item, dict):
                item = Lease().from_dict(item)
            cleaned.append(item)
        self._properties['items'] = cleaned

    @property
    def metadata(self) -> 'ListMeta':
        """
        Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/api-
        conventions.md#metadata
        """
        return self._properties.get('metadata')

    @metadata.setter
    def metadata(self, value: typing.Union['ListMeta', dict]):
        """
        Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/api-
        conventions.md#metadata
        """
        if isinstance(value, dict):
            value = ListMeta().from_dict(value)
        self._properties['metadata'] = value

    @staticmethod
    def get_resource_api(
            api_client: client.ApiClient = None,
            **kwargs
    ) -> 'client.CoordinationV1beta1Api':
        """
        Returns an instance of the kubernetes API client associated with
        this object.
        """
        if api_client:
            kwargs['apl_client'] = api_client
        return client.CoordinationV1beta1Api(**kwargs)

    def __enter__(self) -> 'LeaseList':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False


class LeaseSpec(_kuber_definitions.Definition):
    """
    LeaseSpec is a specification of a Lease.
    """

    def __init__(
            self,
            acquire_time: 'MicroTime' = None,
            holder_identity: str = None,
            lease_duration_seconds: int = None,
            lease_transitions: int = None,
            renew_time: 'MicroTime' = None,
    ):
        """Create LeaseSpec instance."""
        super(LeaseSpec, self).__init__(
            api_version='coordination/v1beta1',
            kind='LeaseSpec'
        )
        self._properties = {
            'acquireTime': acquire_time if acquire_time is not None else MicroTime(),
            'holderIdentity': holder_identity if holder_identity is not None else '',
            'leaseDurationSeconds': lease_duration_seconds if lease_duration_seconds is not None else None,
            'leaseTransitions': lease_transitions if lease_transitions is not None else None,
            'renewTime': renew_time if renew_time is not None else MicroTime(),

        }
        self._types = {
            'acquireTime': (MicroTime, None),
            'holderIdentity': (str, None),
            'leaseDurationSeconds': (int, None),
            'leaseTransitions': (int, None),
            'renewTime': (MicroTime, None),

        }

    @property
    def acquire_time(self) -> 'MicroTime':
        """
        acquireTime is a time when the current lease was acquired.
        """
        return self._properties.get('acquireTime')

    @acquire_time.setter
    def acquire_time(self, value: typing.Union['MicroTime', dict]):
        """
        acquireTime is a time when the current lease was acquired.
        """
        if isinstance(value, dict):
            value = MicroTime().from_dict(value)
        self._properties['acquireTime'] = value

    @property
    def holder_identity(self) -> str:
        """
        holderIdentity contains the identity of the holder of a
        current lease.
        """
        return self._properties.get('holderIdentity')

    @holder_identity.setter
    def holder_identity(self, value: str):
        """
        holderIdentity contains the identity of the holder of a
        current lease.
        """
        self._properties['holderIdentity'] = value

    @property
    def lease_duration_seconds(self) -> int:
        """
        leaseDurationSeconds is a duration that candidates for a
        lease need to wait to force acquire it. This is measure
        against time of last observed RenewTime.
        """
        return self._properties.get('leaseDurationSeconds')

    @lease_duration_seconds.setter
    def lease_duration_seconds(self, value: int):
        """
        leaseDurationSeconds is a duration that candidates for a
        lease need to wait to force acquire it. This is measure
        against time of last observed RenewTime.
        """
        self._properties['leaseDurationSeconds'] = value

    @property
    def lease_transitions(self) -> int:
        """
        leaseTransitions is the number of transitions of a lease
        between holders.
        """
        return self._properties.get('leaseTransitions')

    @lease_transitions.setter
    def lease_transitions(self, value: int):
        """
        leaseTransitions is the number of transitions of a lease
        between holders.
        """
        self._properties['leaseTransitions'] = value

    @property
    def renew_time(self) -> 'MicroTime':
        """
        renewTime is a time when the current holder of a lease has
        last updated the lease.
        """
        return self._properties.get('renewTime')

    @renew_time.setter
    def renew_time(self, value: typing.Union['MicroTime', dict]):
        """
        renewTime is a time when the current holder of a lease has
        last updated the lease.
        """
        if isinstance(value, dict):
            value = MicroTime().from_dict(value)
        self._properties['renewTime'] = value

    def __enter__(self) -> 'LeaseSpec':
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False



import datetime as _datetime

from kuber import versioning

# Information about the kubernetes API library version associated with this
# package and kuber metadata around its creation and labeling within the
# kuber library.
KUBERNETES_VERSION = versioning.KubernetesVersion(
    label='v1.12',
    version='v1.12.10',
    major='1',
    minor='12',
    patch='10',
    pre_release='',
    build='',
    created_at=_datetime.datetime(2020, 11, 23),
    commit_sha='e3c134023df5dea457638b614ee17ef234dc34a6'
)



import os

import kuber
from kuber.latest import core_v1

directory = os.path.realpath(os.path.dirname(__file__))


def test_odd_api_versions():
    """Should serialize and deserialize odd api versions."""
    bundle = kuber.from_file(
        os.path.join(directory, 'same-names-different-resources.yaml'),
        kubernetes_version='latest'
    )

    service_account: core_v1.ServiceAccount = (
        bundle.resources.service_account.foo_bar
    )
    assert service_account.kind == 'ServiceAccount'



import os
import re

from setuptools import find_packages
from setuptools import setup

MY_DIRECTORY = os.path.dirname(__file__)


def get_version():
    path = os.path.join(MY_DIRECTORY, 'kuber', '__init__.py')
    with open(path, 'r') as f:
        return (
            re.compile(r'\n__version__\s+=\s+\'(?P<version>[0-9.]+)\'')
            .search(f.read())
            .group('version')
        )


def readme():
    path = os.path.realpath(os.path.join(
        os.path.dirname(__file__),
        'README.md'
    ))
    with open(path) as f:
        return f.read()


def populate_extra_files():
    """
    Creates a list of non-python data files to include in package distribution
    """
    return []


setup(
    name='kuber',
    version=get_version(),
    description=(
        'High-level Kubernetes resource '
        'configuration and management library.'
    ),
    long_description=readme(),
    long_description_content_type='text/markdown',
    keywords=['kubernetes', 'containers', 'kubectl', 'k8s'],
    url='https://github.com/sernst/kuber',
    author='Scott Ernst',
    author_email='swernst@gmail.com',
    license='MIT',
    platforms='Linux, Mac OS X, Windows',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data={'': populate_extra_files()},
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',

        'Topic :: Software Development :: Libraries',
        'Topic :: System :: Systems Administration'
    ],
    install_requires=['pyyaml', 'kubernetes'],
    extras_require={},
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-cov']
)



import sys
import os
import subprocess
import shutil

MY_DIRECTORY = os.path.realpath(os.path.dirname(__file__))
ROOT_DIRECTORY = os.path.realpath(os.path.join(MY_DIRECTORY, '..'))


def deploy_release():
    """Deploys the current version to PyPi."""
    print('\n\n=== BUILD & DEPLOY RELEASE ===\n')
    directory = os.path.join(ROOT_DIRECTORY, 'dist')
    if os.path.exists(directory):
        shutil.rmtree(directory)

    os.chdir(ROOT_DIRECTORY)

    cmd = [sys.executable, 'setup.py', 'sdist', 'bdist_wheel']
    subprocess.run(cmd).check_returncode()

    cmd = ['twine', 'upload', 'dist/kuber*']
    os.system(' '.join(cmd))

    print('\nOperation Complete.\n\n')



import json
import textwrap
import typing

import yaml
from kubernetes.client.rest import ApiException

from kuber import definitions

CREATE_SYMBOLS = {
    'AlreadyExists': '=',
    'NotFound': '!!'
}

STATUS_SYMBOLS = {
    'NotFound': ' '
}

DELETE_SYMBOLS = {
    'NotFound': ' '
}


class ResponseInfo(typing.NamedTuple):
    """Data structure for parsed ApiExceptions."""

    resource: 'definitions.Resource'
    symbol: str
    reason: str
    message: str = None
    exception: typing.Optional[ApiException] = None
    status: typing.Optional['definitions.Definition'] = None


def _parse_api_exception(
        resource: 'definitions.Resource',
        error: ApiException,
        symbols: dict = None
) -> ResponseInfo:
    """Returns the parsed body of the given error"""
    try:
        body = json.loads(error.body)
    except json.JSONDecodeError:
        body = {}

    reason = body.get('reason', 'UnknownError')
    return ResponseInfo(
        resource=resource,
        symbol=(symbols or {}).get(reason, '!!'),
        reason=reason,
        message='{}'.format(body.get('message', error)),
        exception=error
    )


def _echo_response(response: ResponseInfo):
    """Displays the response for human consumption."""
    resource_id = '{}/{}'.format(
        response.resource.kind,
        response.resource.metadata.name or response.resource.kuber_uid
    )
    wrap = '\n' if response.symbol == '!!' else ''
    print(f'{wrap}[{response.symbol}] {response.reason}: {resource_id}{wrap}')

    if response.symbol not in ('!!', '*'):
        return

    message = '\n'.join(textwrap.wrap(response.message, 70))
    print(textwrap.indent(message, prefix='   '), end='\n\n')

    if response.symbol == '!!':
        print(
            textwrap.indent(response.resource.to_yaml(), prefix='   '),
            end='\n\n'
        )


def create_resource(
        resource: 'definitions.Resource',
        namespace: str = None,
        echo: bool = False
) -> ResponseInfo:
    """..."""
    try:
        status = resource.create_resource(namespace)
        response = ResponseInfo(resource, '+', 'Created', status=status)
    except ApiException as error:
        response = _parse_api_exception(resource, error)

    if response.reason == 'AlreadyExists':
        return patch_resource(resource, namespace, echo)

    if echo:
        _echo_response(response)
    return response


def replace_resource(
        resource: 'definitions.Resource',
        namespace: str = None,
        echo: bool = False
) -> ResponseInfo:
    """..."""
    try:
        status = resource.replace_resource(namespace)
        response = ResponseInfo(resource, '-/+', 'Replaced', status=status)
    except ApiException as error:
        response = _parse_api_exception(resource, error)

    if echo:
        _echo_response(response)
    return response


def patch_resource(
        resource: 'definitions.Resource',
        namespace: str = None,
        echo: bool = False
) -> ResponseInfo:
    """..."""
    try:
        status = resource.patch_resource(namespace)
        response = ResponseInfo(resource, '~', 'Updated', status=status)
    except ApiException as error:
        response = _parse_api_exception(resource, error)

    if echo:
        _echo_response(response)
    return response


def get_resource_status(
        resource: 'definitions.Resource',
        namespace: str = None,
        echo: bool = False
) -> ResponseInfo:
    """Shows statuses of resources in the bundle."""
    try:
        status: definitions.Definition = resource.get_resource_status(
            namespace=namespace
        )
        if status is not None and status.to_dict():
            message = yaml.dump(status.to_dict(), default_flow_style=False)
            response = ResponseInfo(
                resource=resource,
                symbol='*',
                reason='Status',
                message=message,
                status=status
            )
        else:
            status = resource.read_resource(namespace=namespace)
            response = ResponseInfo(
                resource=resource,
                symbol='*',
                reason='Status',
                message='Exists in cluster and is behaving as expected.',
                status=status
            )
    except ApiException as error:
        response = _parse_api_exception(resource, error)

    if echo:
        _echo_response(response)
    return response


def delete_resource(
        resource: 'definitions.Resource',
        namespace: str = None,
        echo: bool = False
) -> ResponseInfo:
    """..."""
    try:
        status = resource.delete_resource(namespace)
        response = ResponseInfo(resource, '-', 'Deleted', status=status)
    except ApiException as error:
        response = _parse_api_exception(resource, error)

    if echo:
        _echo_response(response)
    return response
